/*
 * Copyright (c) 2007 Kathryn Huxtable
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: UserAttributes.java 1248 2007-04-18 12:00:08Z kathryn5 $
 */
package org.kathrynhuxtable.middleware.shibshim.filter;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Set;

/**
 * Public interfact for the attributes object for the Shibboleth shim filter.
 * Hold the collection of attributes and provide parsing method.
 */
public interface UserAttributes {

    /**
     * Returns <tt>true</tt> if this map contains a mapping for the specified
     * key. More formally, returns <tt>true</tt> if and only if this map
     * contains a mapping for a key <tt>k</tt> such that
     * <tt>(key==null ? k==null : key.equals(k))</tt>. (There can be at most
     * one such mapping.)
     * 
     * @param key
     *            key whose presence in this map is to be tested.
     * @return <tt>true</tt> if this map contains a mapping for the specified
     *         key.
     * @throws ClassCastException
     *             if the key is of an inappropriate type for this map
     *             (optional).
     * @throws NullPointerException
     *             if the key is null and this map does not permit null keys
     *             (optional).
     */
    boolean containsKey(Object key) throws ClassCastException, NullPointerException;

    /**
     * Returns <tt>true</tt> if this map maps one or more keys to the
     * specified value. More formally, returns <tt>true</tt> if and only if
     * this map contains at least one mapping to a value <tt>v</tt> such that
     * <tt>(value==null ? v==null : value.equals(v))</tt>.
     * 
     * @param value
     *            value whose presence in this map is to be tested.
     * @return <tt>true</tt> if this map maps one or more keys to the
     *         specified value.
     * @throws ClassCastException
     *             if the key is of an inappropriate type for this map
     *             (optional).
     * @throws NullPointerException
     *             if the key is null and this map does not permit null keys
     *             (optional).
     */
    boolean containsValue(Object value) throws ClassCastException, NullPointerException;

    /**
     * Returns a set view of the mappings contained in this map. Each element in
     * the returned set is a <code>Map.Entry</code>. The set is backed by the
     * map, so changes to the map are reflected in the set, and vice-versa. If
     * the map is modified while an iteration over the set is in progress
     * (except through the iterator's own <tt>remove</tt> operation, or
     * through the <tt>setValue</tt> operation on a map entry returned by the
     * iterator) the results of the iteration are undefined. The set supports
     * element removal, which removes the corresponding mapping from the map,
     * via the <tt>Iterator.remove</tt>, <tt>Set.remove</tt>,
     * <tt>removeAll</tt>, <tt>retainAll</tt> and <tt>clear</tt>
     * operations. It does not support the <tt>add</tt> or <tt>addAll</tt>
     * operations.
     * 
     * @return a set view of the mappings contained in this map.
     */
    Set entrySet();

    /**
     * Returns the value to which this map maps the specified key. Returns
     * <tt>null</tt> if the map contains no mapping for this key. A return
     * value of <tt>null</tt> does not <i>necessarily</i> indicate that the
     * map contains no mapping for the key; it's also possible that the map
     * explicitly maps the key to <tt>null</tt>. The <tt>containsKey</tt>
     * operation may be used to distinguish these two cases.
     * 
     * <p>
     * More formally, if this map contains a mapping from a key <tt>k</tt> to
     * a value <tt>v</tt> such that <tt>(key==null ? k==null :
     * key.equals(k))</tt>,
     * then this method returns <tt>v</tt>; otherwise it returns
     * <tt>null</tt>. (There can be at most one such mapping.)
     * </p>
     * 
     * @param key
     *            key whose associated value is to be returned.
     * @return the value to which this map maps the specified key, or
     *         <tt>null</tt> if the map contains no mapping for this key.
     */
    Object get(Object key);

    /**
     * Retrieve an attribute, returning only one value if multiple values are
     * present.
     * 
     * @param name
     *            the name of the parameter.
     * @return attribute value for the parameter.
     */
    String getHeader(String name);

    /**
     * Return attribute values as an Enumeration.
     * 
     * @param name
     *            the name of the parameter.
     * @return attribute values for the parameter.
     */
    Enumeration getHeaders(String name);

    /**
     * Return Enumeration of attribute names.
     * 
     * @return attribute name Enumeration
     */
    Enumeration getHeaderNames();

    /**
     * Determine if this is a mapped attribute or a special attribute.
     * 
     * @param name
     *            the attribute to test.
     * @return true if the name is a mapped attribute.
     */
    boolean isMappedAttribute(String name);

    /**
     * @return <tt>true</tt> if this map contains no key-value mappings.
     */
    boolean isEmpty();

    /**
     * Returns a set view of the keys contained in this map. The set is backed
     * by the map, so changes to the map are reflected in the set, and
     * vice-versa. If the map is modified while an iteration over the set is in
     * progress (except through the iterator's own <tt>remove</tt> operation),
     * the results of the iteration are undefined. The set supports element
     * removal, which removes the corresponding mapping from the map, via the
     * <tt>Iterator.remove</tt>, <tt>Set.remove</tt>,
     * <tt>removeAll</tt> <tt>retainAll</tt>, and <tt>clear</tt>
     * operations. It does not support the add or <tt>addAll</tt> operations.
     * 
     * @return a set view of the keys contained in this map.
     */
    Set keySet();

    /**
     * @return Returns the number of key-value mappings in this map. If the map
     *         contains more than <tt>Integer.MAX_VALUE</tt> elements, returns
     *         <tt>Integer.MAX_VALUE</tt>.
     */
    int size();

    /**
     * Returns a collection view of the values contained in this map. The
     * collection is backed by the map, so changes to the map are reflected in
     * the collection, and vice-versa. If the map is modified while an iteration
     * over the collection is in progress (except through the iterator's own
     * <tt>remove</tt> operation), the results of the iteration are undefined.
     * The collection supports element removal, which removes the corresponding
     * mapping from the map, via the <tt>Iterator.remove</tt>,
     * <tt>Collection.remove</tt>, <tt>removeAll</tt>, <tt>retainAll</tt>
     * and <tt>clear</tt> operations. It does not support the add or
     * <tt>addAll</tt> operations.
     * 
     * @return a collection view of the values contained in this map.
     */
    Collection values();
}
